/*
 * Gir vi oss selv presisjons-poeng for  ikke finne noe som ikke finnes  ?
 * Dvs, hvis GENIA er enige med oss i at en event ikke har en cause eller theme ?
 *
 * Det hres ut som et ml p hvor god mappingen av TUC-events til (de noe frre) GENIA-events er. Det er sikkert lurt  ha med dette tallet, men ogs her er det mulighet for  mle bde presisjon og recall.
 * 
 * Dvs: Hvor ofte finner vi rett mapping (TP), gal mapping (FP), ingen mapping (TN), og manglende mapping/TUCevent til gitt GENIA-event (FN).
 *  
 *   Hvis vi skifter granulariteten litt kan vi da se p kun matchende TP-events, og telle themes istedenfor events, for  finne nye P, R og F-verdier med den granulariteten.
 *
 *
 *   Presisjon er TP/(TP+FP) = TP/(All guesses)
 *   Recall er TP/(TP+FN) = TP/(All actual events)
 *
 *
 *
 */

import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.InputStreamReader;


/**
 * This class performs evaluations of the events affilated with a sentence. It is normally instantiated from EvEx. The justification for this class is documented in the main report, chapter "Implementation". 
 * Definition: Precision = TP/(FP+TP) = TP/(All guesses)
 * Definition: Recall = TP/(TP+FN) = TP/(All actual events)
 * Definition: F-score = 2*P*R/(P+R)
 *
 * @see Anto
 * @see Syno
 * @see GSentence
 * @see TQL
 * @see XMLWriter
 * @see EvEx
 *
 */

public class Evaluator {


	public GSentence g;
	public TQL t;
	public ArrayList g_events;
	public ArrayList t_events;
	public EvEx parent;
	public Syno syno;
	public Anto anto;
	public XMLWriter xw;
	public boolean debug_eval =  false;

	/*
	 * Evaluation scores
	 */

	public int g_total;	
	public int t_total;	

	public int t_type_correct;
	public int t_theme_correct;
	public int t_cause_correct;
	public int t_all_correct;

	public int t_type_total;
	public int t_theme_total;
	public int t_cause_total;

	/** 
	 * Create a new evaluation for the sentence defined by GSentence and TQL.
 	 * @param g the sentence from GOLD
 	 * @param t the sentende from TEST
 	 * @param syno the synonyme class
 	 * @param anto the antonyme class
 	 * @param xw the writer for outputting distilled events
 	 */
	public Evaluator(GSentence g, TQL t, EvEx parent, Syno syno, Anto anto, XMLWriter xw){
		this.g = g;
		this.g_events = g.parent.getEvents();
		this.t = t;
		this.parent = parent;
		this.syno = syno;
		this.anto = anto;
		this.xw = xw;
		t_events = t.getEvents();
		eval();
	}

	/**
	 * Wrapper method to start evaluation.
	 */
	public void eval() {
		evalAuto();
	}

	/**
	 * Method to evaluate the events in a given sentence.
	 * It also takes care of writing distilled events to defined xml-files.
	 * An (test) event is evaluated only if it matches the (gold) event.
	 * <ul>A match is defined by: <li>equal type, or <li> no equal types in later events, leaving rest of test.size() >= gold.size()</ul>
	 * If no equality between types are found (lexical equality or syno/anto), the user is queried when --manual is given at command line.
	 * When no other possibility for equality is found, the event are positively registeded to the recall score, but does not affect precision.
	 *
	 * Evaluation results is stored as member variables to be accessed from EvEx later.
	 *
	 * This method also wraps the now obsolete evalManual()
	 * 
	 */
	public void evalAuto() {

		
		g_events =  g.getEvents();
		t_events =  t.getEvents();

		// add total number
		g_total = g_events.size();	
		t_total = t_events.size();	

		if ( debug_eval ) System.out.println("\n== evaluating sentence "+t.input+" ("+t_total+" vs "+g_total+" events)");

		// write events and dummy events to xml-out
		for ( int i = 0 ; (i < g_total) || (i < t_total) ; i ++ ) {
			if ( i < t_total ) {
				Event te = (Event) t_events.get(i);			  		
				xw.writeTQLEvent(te);
			} else {
				xw.writeTQLEvent(new Event());
			}

			if ( i < g_total ) {
				GEvent ge = (GEvent) g_events.get(i);			  		
				xw.writeGENEvent(ge);
			} else {
				xw.writeGENEvent(new GEvent());
			}
		}	  

		int g = 0;
		int t = 0;

		// iterate over all guessed events
		while ( t < t_total) {

			Event te  = (Event)  t_events.get(t);

			// count guesses
			t_type_total++;
			t_theme_total++;
			t_cause_total++;

         if (g < g_total) {

			   GEvent ge = (GEvent) g_events.get(g);

            // count corrects 
			   if ( isHit(ge.getRealType(),  te.getType()) ) {
			   	if ( isHit(ge.getRealType(),  te.getType()  ) ) t_type_correct++; 	
			   	if ( isHit(ge.getRealTheme(), te.getTheme() ) ) t_theme_correct++; 	
			   	if ( isHit(ge.getRealCause(), te.getCause() ) ) t_cause_correct++; 	

			   } else if ( isAhead(te.getType(), g) ) {
					// we skip ahead if:
					// 1. there is a matching type, nad
					// 2. there are at least as many GE left as TE after the skip
					
					int skipvalue = g_total - getTypeAhead(te.getType(), g);
					// System.out.println("Will skip ahead, remaing TE:"+t_total +">="+ skipvalue);

					if (skipvalue > t_total) {	
						g = getTypeAhead(te.getType(), g);
						ge = (GEvent) g_events.get(g);

						// check for faulty skip - not essential
						if ((t+1 < t_total) && (g+1 >= g_total) ){
							System.out.println("== ERROR: heuristic skip caused T-event overflow");
							System.exit(1);
						}

					} 

			   	if ( isHit(ge.getRealType(),  te.getType()  ) ) t_type_correct++; 	
			   	if ( isHit(ge.getRealTheme(), te.getTheme() ) ) t_theme_correct++; 	
			   	if ( isHit(ge.getRealCause(), te.getCause() ) ) t_cause_correct++; 	

			   } else {	
					// we can do no better than to carry on
			   	if ( isHit(ge.getRealType(),  te.getType()  ) ) t_type_correct++; 	
			   	if ( isHit(ge.getRealTheme(), te.getTheme() ) ) t_theme_correct++; 	
			   	if ( isHit(ge.getRealCause(), te.getCause() ) ) t_cause_correct++; 	
			   	t++;	
			   }

            // count all corrects 
			   if ( isHit(ge.getRealType(),  te.getType()) && isHit(ge.getRealType(),  te.getType()) && isHit(ge.getRealTheme(), te.getTheme()) ) t_all_correct++;

			   // System.out.println( "muu: "+ ge.get("real_theme")+ " : " + ge.getTheme() );

			   if ( debug_eval ) echoEvaluation(ge, te);

			   g++;
			   t++;
			} else {
         t++;
		   }
	   }
	}




	/**
	 * Wrapper method to ensure backward compatibility with evalManual();
	 * Invokes intervention();
	 * @see #intervention(String, String)
	 */
	public boolean isHit(String ge, String te){
		return intervention(ge,te);
		/*	
		// no need to intervene with equal strings
		if ( ( ge == null ) && ( te == null ) ) {
			return true;
		} else if ( ( ge == null ) || ( te == null ) ) {
			return false;
		} else if ( ge.equals(te) ) {
			return true;	
		} else if ( syno.isSyno(ge,te) ) {
			return true;	
		} else if ( anto.isAnto(ge,te) ) {
			return false;	
		} else {
			return false;
		}
		*/
	}

	/**
	 * Simple method to echo the evaulation to stdout
	 */
	public void echoEvaluation(GEvent ge, Event te) {
		System.out.println("\n== RESULT FOR EVALUATED EVENT (TE: "+te.id+" GE: "+ge.id+")");
		System.out.println( ge.get("real_typeclass")+" == "+te.getType()   +"\t("+isHit(ge.get("real_typeclass"),te.getType())+")");	
		System.out.println( ge.get("real_theme")+		" == "+te.getTheme()  +"\t("+isHit(ge.get("real_theme"),te.getTheme())+")"); 	
		System.out.println( ge.get("real_cause")+		" == "+te.getCause()  +"\t("+isHit(ge.get("real_cause"),te.getCause())+")"); 	
	}

	/**
	 * Method to determine if two strings are "equal".
	 * 1: if both string are "null", they are "equal"
	 * 2: if one of them are "null", they are not "equal"
	 * 3: if they are lexically equal, they are "equal"
	 * 4: if they are defined synonymes, they are "equal"
	 * 5: if they are defined antonymes,  they are not "equal"
	 * 6: if --manual is supplied, the user gets  to decide if they are "equal"
	 * 7: else they are not "equal"
	 *
	 * If user inputs a decision, it is stored to Anto or Syno for later usage.
	 *
	 * @param ge the GOLD word
	 * @param te the TEST word
	 *
	 * @see Syno
	 * @see Anto
	 */
	public boolean intervention ( String ge, String te ) {

		// if ( debug_eval ) System.out.println("?: "+ge+" == "+te);

		// no need to intervene with equal strings
		if ( ( ge == null ) && ( te == null ) ) {
			return true;
		} else if ( ( ge == null ) || ( te == null ) ) {
			return false;
		} else if ( ge.equals(te) ) {
			return true;	
		} else if ( syno.isSyno(ge,te) ) {
			return true;	
		} else if ( anto.isAnto(ge,te) ) {
			return false;	
		} else if ( parent.manual ) {
		
		   System.out.println("\ninput: "+ge+" == "+te+" ? (N/y/0/1)");
		   BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		   

		   int i = 0;

		   try {
		   	i = in.read();	
		   } catch ( Exception e ) {
		   	e.printStackTrace();
		   }
		   // System.out.println("scancode: "+i);

		   if ( i == 49 || i == 121 ) {
		   	syno.setSyno(ge,te);
		   	return true;
		   } else if ( i == 48 || i == 10 || i == 110 ) {
		   	anto.setAnto(ge,te);
		   	return false;
		   //} else if ( i == 97 ) {
		   //	System.out.println("== skipping all user inputs");
		   //	parent.manual = true;
		   } else {
		   	System.out.println("Use (return, 0, n) for false or (1, y) for true. Default is: false");
		   	intervention(ge,te);
		   }
		   return false;

		} else {

			return false;

		}

	}
	
	/**
	 * Simple method for echoing an evaluation.
	 */
	public void hello() {
      System.out.println("== EVALUTAION ==");
      System.out.println("Total events: GE="+g_total +", TE="+t_total);
      System.out.println("Correct 		\ttype="+t_type_correct+", theme="+t_theme_correct+", cause="+t_cause_correct+", all="+t_all_correct);  
      System.out.println("Total 			\ttype="+t_type_total+", theme="+t_theme_total+", cause="+t_cause_total);  
      System.out.println("\n");  
	}

	/**
	 * Assisting method to determine is an event type is ahead in the golden array of events.
	 */
	public boolean isAhead(String t_type, int g) {

      while ( g < g_events.size() ) {
			GEvent ge = (GEvent) g_events.get(g);
			String g_type = ge.getRealType();
			// System.out.println("Looking ahead at: "+g_type+" = "+t_type);
         if ( isHit(g_type, t_type) ) {
				// System.out.println("returning true: "+g_type+" = "+t_type);
			   return true;
			} else {
  				g++;
			}
		}

		//System.out.println("returning false");
		return false;
	}

	/**
	 * Assisting method to determine how far ahead one have to skip to get to a given type in the golden array.
	 */
	public int getTypeAhead(String t_type, int g) {

		int old_g = g;

      while ( g < g_events.size() ) {
			GEvent ge = (GEvent) g_events.get(g);
			String g_type = ge.getRealType();
         if ( isHit(g_type, t_type) ) {
			   return g;
			} else {
  				g++;
			}
		}
		// should never be reached, but implements safe traversal og GEvents
		return old_g;
	}

}
