import java.util.ArrayList;
import java.util.Hashtable;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * This class handles the content of XML in GOLD-files. For each tag, there is a definition of action to take. This is usually related to putting the attribute data of the tag somewhere.
 * @see GENIAParser
 */
public class GENIAHandler extends DefaultHandler {

	public ArrayList<GAbstract> abstracts = new ArrayList<GAbstract>();
	public GAbstract gabstract; 
	public GSentence gs; 
	public GEvent ge; 
	public String lastTag;
	public int sentencecounter = 1;

	public boolean debug_geh = false;

	/**
	 * This class does not need anything in particular to be instantiated.
	 */
	public GENIAHandler(){
	}

	/**
	 * Invoked when a document is about to be parsed.\ Does nothing.
	 */
	public void startDocument(){
	}

	/**
	 * Invoked when a document is finished parsed. Does nothing but informing the user about this happening.
	 */
	public void endDocument(){
		System.out.println("== goldfile parsing complete");
	}

	/**
	 * Called when the parser find a new starting tag. This tag is matched against a number of clauses, i.e. sentence, term, event .. If no mathcing clause is found, the tag is ignored.
	 * This method sets data according to which tag has been discovered.
	 * @see #endElement(String, String, String)
	 */
	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
		lastTag = qName;

		if(qName == "PMID"){
			
			gabstract = new GAbstract();

		} else if(qName == "sentence"){
			
			gs = new GSentence(gabstract);
			gs.setID(attrs.getValue("id"));
			gabstract.addSentence(gs);	


		} else if (qName == "term") {
			GTerm gt = new GTerm(gs);
			gt.setID(  attrs.getValue("id"));

			String lex = attrs.getValue("lex");
			String sem = attrs.getValue("sem");

			if ( lex == null ) {
				gt.setlex(sem);
			} else {
				gt.setlex(lex);
			}
			gt.setsem(sem);
			gs.addTerm(gt);
	
		} else if (qName == "event") {
			ge = new GEvent();
			ge.setID(attrs.getValue("id"));
			// ge.sentence = sentencecounter;
			ge.sentence = gs.id;	
			gs.addEvent(ge);
			gabstract.addEvent(ge);

		} else if (qName == "type") {
			ge.setType(attrs.getValue("class"));

		} else if (qName == "theme") {
			ge.setTheme(attrs.getValue("idref"));
			if ( debug_geh ) System.out.println("Event "+ge.id+" theme = "+ge.getTheme());

		} else if (qName == "cause") {
			ge.setCause(attrs.getValue("idref"));

		} else if (qName == "PubmedArticle") {
				  gabstract = new GAbstract();
		} else {

		}

   }

	/**
	 * This method completes the action of startElement, namely unsets references that has been set, and adding items to misc lists.
	 * @see #startElement(String, String, String, Attributes)
	 */
	public void endElement(String namespaceURI, String sName, String qName ) throws SAXException {
		lastTag = null; 

		if(qName == "sentence"){
			// gs = null; - need last sentence to add events
		} else if (qName == "event") {
			ge = null;
		} else if (qName == "term") {
			// do noting ?
		} else if (qName == "PubmedArticle") {
				  // finished building abstract
				  gabstract.buildTerms();
				  abstracts.add(gabstract);
				  gabstract = null;
		} else {
		}
   }

	/**
	 * The string between tags (usually empty) is processed by this method. It is only used to to set PMID of abstracts.
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		String s = new String(buf, offset, len);
		s = s.trim();
		emit(s);	

		if(lastTag == "PMID"){

			gabstract.setPMID(s);

		} else if (lastTag == "clueType") {
		
			emit("clueType: "+s);		  	
		/* code for printing elements
        	String s = new String(buf, offset, len);
		s = s.trim();
        	if(s.length() > 0) emit("CHAR: |"+s+"|");
		*/
   	}
	}

	public void emit(String s) throws SAXException {
		// System.out.println(s);
	}

	/**
	 * Accessor method for getting the abstracts handled by this handler.
	 * @return an arraylist of abstracts
	 */
	public ArrayList getAbstracts(){
		return abstracts;
	}

}
