import java.util.ArrayList;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * This class invokes the parsing of a GOLD-document.
 * @see GENIAHandler
 */
public class GENIAParser {

	public File file;
	public GENIAHandler handler;

	public GENIAParser(File file){
		this.file = file;
		parseGENIA(file);
	}

	/**
	 * Creates a new parser, and starts this with the actual document.
	 * @param file the xml-file containing the golden corpus
	 */
	public void parseGENIA(File file){

		try {
			handler = new GENIAHandler();
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( file , handler );
		} catch ( Exception e ) {
			e.printStackTrace();
			System.exit(1);
		}

	}

	/**
	 * Access method to get the handler of this parser.
	 * @return the handler of this document
	 */
	public DefaultHandler getHandler(){
		return handler;
	}
	/**
	 * Access method to get the abstracts parsed by this parser.
	 * @return a list of abstracts parsed by this parser
	 */
	public ArrayList getAbstracts(){
		return handler.getAbstracts();
	}


}
