import java.util.ArrayList;
import java.util.Hashtable;

/**
 *  This class represents a sentence from GENIA golden corpus. An exammple of such sentence is shown below:
 *  
 *&lt;sentence id="S1"&gt;<br>
 * Activation of <br>
 * &lt;term id="T1" lex="NF-kappa_B" sem="Protein_molecule"&gt;<br>
 *  NF-kappa B<br>
 * &lt;/term&gt; <br>
 *  by <br>
 * &lt;term id="T2" lex="interleukin_2" sem="Protein_molecule"&gt;<br>
 *  interleukin 2<br>
 * &lt;/term&gt; <br>
 * in <br>
 * &lt;term id="T3" lex="human_blood_monocyte" sem="Cell_type"><br>
 *  &lt;term id="T4" lex="human" sem="Multi_cell"&gt;<br>
 *   human<br>
 *  &lt;/term&gt; <br>
 *  blood <br>
 *  &lt;term id="T5" lex="monocyte" sem="Cell_type"&gt;<br>
 *  monocytes<br>
 * &lt;/term&gt;<br>
 * &lt;/term&gt;<br>
 * .<br>
 *&lt;/sentence&gt;<br>
 *
 * @see GAbstract
 * @see GTerm
 */

public class GSentence {

	public String id;
	public ArrayList<GTerm> terms = new ArrayList<GTerm>();
	public ArrayList<GEvent> events = new ArrayList<GEvent>();
	public GAbstract parent;
	public int sentence_number;


	/**
	 * To construct an GSentence, it has to belong to an abstract.
	 */
	public GSentence(GAbstract p){
		this.parent = p;
	}

	/**
	 * Set the ID of this sentence.
	 * @param i string representing the id of this sentence
	 */
	public void setID(String i) {
			  id = i;
	}

	/**
	 * Add a GTerm to this sentence.
	 * @param gt the term to be added
	 */
	public void addTerm(GTerm gt) {
		terms.add(gt);
	}

	/**
	 * Add a GEvent to this sentence.
	 * @param ge the event to be added
	 */
	public void addEvent(GEvent ge){
			events.add(ge);
	}

	/**
	 * Get the events associated with this sentence.
	 * @return the events in this sentence
	 */
	public ArrayList getEvents(){
		return events;
	}

	/**
	 * Echo the events of this sentence to stdour.
	 */
	public void printEvents(){
		for(GEvent g : events){
			g.hello();
		}
	}

	/**
	 * Wrapper for printEvents()
	 * @see #printEvents()
	 */
	public void hello(){
		printEvents();
	}

	/**
	 * Echo the terms of this sentence to stdout.
	 */
	public void printTerms(){
		System.out.println("Sentence ID: "+id);
		for(GTerm gt : terms){
			gt.hello();
		}
	}

}
