import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * This class is assisting TQL in the extraction of events. It holds a list of words that are of some importance ("buzzwords").
 * Such buzzword may have significance when discovered in a cruical position in TQL.
 * @see TQL
 */
public class OntologyParser extends DefaultHandler {

	public File file;
	public OntologyParser handler;
	public boolean subclass = false;
	public String currClass = null;
	public HashMap<String,String> onto = new HashMap<String,String>();

	public boolean debug_op = false;
	public boolean debug_op_2 = false;

	// Train
	// public String[] clues = { "abolished", "absence", "accompanied", "accumulation", "activated", "activates", "activating", "activation", "Activation", "activation induced", "activator", "acts", "allows transactivation", "a role", "attenuates", "barely modified", "be", "binding", "binding activity", "binds", "block", "blocks", "cause", "caused", "changes", "component", "composed", "concomitant", "constitute", "contains", "deleted", "deletion", "demonstrate normal", "dependent", "depending", "detected", "directed", "disappearance", "disrupted", "disrupts", "elim", "engage", "enhanced", "enhancing", "essential", "events", "exclusion", "exert", "expression", "fails to stimulate", "following", "for full activation", "inate", "increase", "increased", "increasing", "independent", "induce", "induced", "inducibility", "induction", "induction of", "infected", "infection", "influences", "inhibit", "inhibition", "inhibitor", "inhibitory function", "inhibits", "interaction", "involved", "involvement", "latent phases", "lead", "leading", "leads", "localization", "masks", "mediated", "mediated inhibition", "mediator", "modification toward reduction", "no loss", "not", "not induced", "observed", "perpetuating loop", "perpetuation", "play", "prerequisite", "prevents", "production", "progression", "provides", "rate", "receptor", "reduced", "reduces", "regulate", "regulated", "regulating", "regulation", "regulatory interaction", "replication", "required", "required to activate", "requires", "response", "results", "retarget", "retargeting", "scavenging", "Secretion", "sequences", "serves as", "serves to", "subsequent", "subsequently", "sufficient", "suppress", "sustained", "synthesis", "system", "through", "to inhibit", "transactivation", "trans activation", "transcription", "transcriptional activation", "transcript levels", "translocation", "Treatment", "unaffected", "unchanged", "underlies", "upregulation" };

	// Test
	// public String[] clues = { "act", "act in", "act in concert", "activate", "activated", "activation", "Activation", "affect", "affinities", "as", "bind", "binding", "binding", "combination", "complexes", "concert", "contributes", "contribute to", "contribute to regulation", "control", "cooperation", "correlate", "defective recruitment", "deficiency", "dependent", "depends", "dimerization", "directed", "dissociation", "effect", "establish", "expression", "facilitated", "for", "form", "functions in concert", "Heterodimerization", "implicated", "increases", "independent", "induced", "induction", "infection", "influx", "inhibited", "inhibits", "interact", "interaction", "interfering", "involved", "Little effect", "mediated", "minimal", "modulated", "negative", "none", "not", "not", "not required", "not stimulate", "participates", "production", "Production", "regulation", "require", "required", "requires", "response", "role", "sensitive", "stimulate", "stimulated", "stimulation", "synthesis", "transactivation", "transcription", "transcriptional activation", "transcriptional control", "transcriptional effects", "translocation", "unchanged", "underlie", "without affecting", "work", "works" };


	// Train && test
	public String[] clues = { "abolished", "absence", "accompanied", "accumulation", "activated", "activates", "activating", "activation", "Activation", "activation induced", "activator", "acts", "allows transactivation", "a role", "attenuates", "barely modified", "be", "binding", "binding activity", "binds", "block", "blocks", "cause", "caused", "changes", "component", "composed", "concomitant", "constitute", "contains", "deleted", "deletion", "demonstrate normal", "dependent", "depending", "detected", "directed", "disappearance", "disrupted", "disrupts", "elim", "engage", "enhanced", "enhancing", "essential", "events", "exclusion", "exert", "expression", "fails to stimulate", "following", "for full activation", "inate", "increase", "increased", "increasing", "independent", "induce", "induced", "inducibility", "induction", "induction of", "infected", "infection", "influences", "inhibit", "inhibition", "inhibitor", "inhibitory function", "inhibits", "interaction", "involved", "involvement", "latent phases", "lead", "leading", "leads", "localization", "masks", "mediated", "mediated inhibition", "mediator", "modification toward reduction", "no loss", "not", "not induced", "observed", "perpetuating loop", "perpetuation", "play", "prerequisite", "prevents", "production", "progression", "provides", "rate", "receptor", "reduced", "reduces", "regulate", "regulated", "regulating", "regulation", "regulatory interaction", "replication", "required", "required to activate", "requires", "response", "results", "retarget", "retargeting", "scavenging", "Secretion", "sequences", "serves as", "serves to", "subsequent", "subsequently", "sufficient", "suppress", "sustained", "synthesis", "system", "through", "to inhibit", "transactivation", "trans activation", "transcription", "transcriptional activation", "transcript levels", "translocation", "Treatment", "unaffected", "unchanged", "underlies", "upregulation" , "act", "act in", "act in concert", "activate", "activated", "activation", "Activation", "affect", "affinities", "as", "bind", "binding", "binding", "combination", "complexes", "concert", "contributes", "contribute to", "contribute to regulation", "control", "cooperation", "correlate", "defective recruitment", "deficiency", "dependent", "depends", "dimerization", "directed", "dissociation", "effect", "establish", "expression", "facilitated", "for", "form", "functions in concert", "Heterodimerization", "implicated", "increases", "independent", "induced", "induction", "infection", "influx", "inhibited", "inhibits", "interact", "interaction", "interfering", "involved", "Little effect", "mediated", "minimal", "modulated", "negative", "none", "not", "not", "not required", "not stimulate", "participates", "production", "Production", "regulation", "require", "required", "requires", "response", "role", "sensitive", "stimulate", "stimulated", "stimulation", "synthesis", "transactivation", "transcription", "transcriptional activation", "transcriptional control", "transcriptional effects", "translocation", "unchanged", "underlie", "without affecting", "work", "works" };
	/**
	 * An ontologyparser with a predefined ontology.
	 * @param filepath path to the ontology file
	 */
	public OntologyParser (String filepath) {
		makeParser(filepath);
	}

	/**
	 * An ontologyparser without a ontology (yet).
	 */
	public OntologyParser () {
	}

	/**
	 * An onologyparser may be started from the command line to verify the correctness of an ontology.\ Thus it has a main-method.
	 */
	public static void main(final String[] args) {
		if ( args.length != 1) {
			System.out.println("usage: java OntologyParser ontology.owl");
			System.out.println("(or invoke as new OntologyParser(String filepath)");
			System.exit(1);
		}

	 	OntologyParser op = new OntologyParser(args[0]);

	}


	/**
	 * Makes a parser, and invokes parsing if there is a defined ontology file.
	 * @param filepath path to the ontology file
	 */
	public void makeParser(String filepath){

		try {
			handler = new OntologyParser();
			file = new File(filepath);
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( file , handler );
		} catch ( Exception e ) {
			e.printStackTrace();
		}

	}

	/**
	 * Checks if a word (noun, verb) is of a certain class in the ontology. If the word is found, but of different class, it recurses upward in the ontology.
	 * @param i the noun or verb
	 * @param c the class
	 * @return a boolean indicating if the noun or verb is of class
	 */
	public boolean isOfClass(String i, String c){

		String p = onto.get(i);

		if ( p == null ) {
			return false;
		} else if ( p.equals(c) ) {
			return true;			
		} else {
			return isOfClass(p,c);
		}

	}

	/**
	 * Wrapper method to high-levelly check if a word has some significance.
	 * @param s the word
	 * @return wether the word is significant or not
	 * @see #isBuzzWord(String)
	 * @see #isClueWord(String)
	 */
	public boolean isInOntology(String s){
			  return onto.containsKey(s) || isBuzzWord(s) || isClueWord(s);
	}

	/**
	 * Wrapper method to mid-levelly check if a word has some significance.
	 * @param s the word
	 * @return wether the word is significant or not
	 * @see #isBuzzNoun(String)
	 * @see #isBuzzVerb(String)
	 */
	public boolean isBuzzWord(String s){
		return isBuzzNoun(s) || isBuzzVerb(s);
	}

	/**
	 * Wrapper method to mid-levelly check if an adjective has some significance.\ This method is tentative. 
	 * @param s the word
	 * @return wether the word is significant or not
	 */
	public boolean isBuzzAdjective(String s){
		if ( debug_op_2 ) System.out.print(" == op query (adjective) : "+s);
		if (	
				s.equals("transcript") || 
				s.equals("binding") 
		   )
		{
			if ( debug_op_2 ) System.out.println(" : true");
			return true;
		} else {
			if ( debug_op_2 ) System.out.println(" : false");
			return false;
	   }
	}

	/**
	 * Wrapper method to low-levelly check if a word has some significance.
	 * @param s the word
	 * @return wether the word is significant or not
	 */
	public boolean isBuzzVerb(String s){

		if ( debug_op_2 ) System.out.print(" == op query (verb) : "+s);

		if (
				// s.equals("abolish") ||
				s.equals("accompany") ||
				// s.equals("accumulate") ||
				s.equals("bind") ||
				s.equals("block") ||
				// s.equals("cause") ||
				// s.equals("compose") ||
				s.equals("develop") ||
				s.equals("disrupt") ||	// CH-B
				s.equals("enhance") ||
				s.equals("increase") || 
				s.equals("induce") || 
				s.equals("influences") ||
				s.equals("inhibit") ||
				// s.equals("result") ||
				s.equals("modify") 
			)
		{
			if ( debug_op_2 ) System.out.println(" : true");
			return true;
		} else {
			if ( debug_op_2 ) System.out.println(" : false");
			return false;
	   }
	}

	/**
	 * Wrapper method to low-levelly check if a word has some significance.
	 * @param s the word
	 * @return wether the word is significant or not
	 */
	public boolean isBuzzNoun(String s){
		if ( debug_op_2 ) System.out.print(" == op query (noun) : "+s);
		if(	s.equals("aaaaaaaaaaaaa") ||
				// s.equals("activation") ||
				// s.equals("accumulation") ||
				// s.equals("absence") ||
				s.equals("activity") ||
				// s.equals("attenuates") ||
				// s.equals("addition") ||
				s.equals("cell_line") ||
				s.equals("coding") ||
				// s.equals("compound") ||
				// s.equals("component") ||
				// s.equals("concomitant") ||
				s.equals("dna_familiy") ||
				// s.equals("effect") ||
				// s.equals("event") ||
				s.equals("enhancement") ||
				// s.equals("exposure") ||
				// s.equals("extract") || // CH-D
				s.equals("factor") ||
				// s.equals("function") ||
				// s.equals("gene") ||
				s.equals("increase") ||
				s.equals("induction") ||
				s.equals("infection") || // CH-A
				s.equals("inhibition") ||
				s.equals("interaction") || // CH-C
				s.equals("level") ||
				// s.equals("observation") ||
				//s.equals("protein_family") ||
				// s.equals("protein_molecule") ||
				s.equals("response") ||
				// s.equals("retargeting") ||
				// s.equals("result") ||
				// s.equals("site") ||
				s.equals("stress") ||
				s.equals("transactivation") ||
				s.equals("translocation") ||
				// s.equals("perpetuation") ||
				s.equals("process") ||
				// s.equals("protein_family") ||
				s.equals("virus") || // CH-E
				s.equals("zzzzzzzzzzzzzz")) {
			if ( debug_op_2 ) System.out.println(" : true");
			return true;
		} else {
			if ( debug_op_2 ) System.out.println(" : false");
			return false;
		}

	}

	/**
	 * Wrapper method to low-levelly check if a word has some significance.
	 * @param s the word
	 * @return wether the word is significant or not
	 */
	public boolean isClueWord(String s){

			  for ( String t : clues ) {

					if ( s.equals(t) ) {
						  return true;
					} 	
			  }
			return false;
	}

	/**
	 * XML-parsing function for start of parsing, but it has no function.
	 */
	public void startDocument(){
		// System.out.println("== starting ontology parsing");
	}

	/**
	 * XML-parsing function for end of parsing.\ It has no function but to inform the user of parser completion.
	 */
	public void endDocument(){
		System.out.println("== ontology parsing complete");

		Set hs = onto.keySet();
		Iterator i = hs.iterator();
	}

	/**
	 * Called when the parser find a new starting tag. This tag is matched against a number of clauses, i.e. sentence, term, event .. If no mathcing clause is found, the tag is ignored.
	 * This method sets data according to which tag has been discovered.
	 * @see #endElement(String, String, String)
	 */
	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
			
			if ( qName == "owl:Class" ) {
				if ( !subclass ) {
					// class definition
					
					if ( ( currClass = attrs.getValue("rdf:ID") ) != null) {
						currClass = rmH(currClass);
						if ( debug_op ) System.out.println("Class: "+currClass);
						if ( ! onto.containsKey ( currClass ) ) onto.put(currClass,null);

					} else if ( ( currClass = attrs.getValue("rdf:about") ) != null) {
						currClass = rmH(currClass);
						if ( debug_op ) System.out.println("Class: "+currClass);
						if ( ! onto.containsKey ( currClass ) ) onto.put(currClass,null);
					}

				} else {
					// subclass specification
					
					String sub;
					if ( ( sub = attrs.getValue("rdf:ID") ) != null ) {
						sub = rmH(sub);
						if ( debug_op ) System.out.println(currClass+ " is subclass of: "+sub);
						if ( onto.containsKey ( currClass ) ) { onto.remove ( currClass ); }
						if ( debug_op ) System.out.println("putting: "+currClass+","+sub);
						onto.put(currClass, sub);
					} else if ( ( sub = attrs.getValue("rdf:resource") ) != null ) {
						sub = rmH(sub);
						if ( debug_op ) System.out.println(currClass+" is subclass of resource: "+sub);
						if ( onto.containsKey ( currClass ) ) { onto.remove ( currClass ); }
						if ( debug_op ) System.out.println("putting: "+currClass+","+sub);
						onto.put(currClass, sub);
					} else if ( ( sub = attrs.getValue("rdf:about") ) != null ) {
						sub = rmH(sub);
						if ( debug_op ) System.out.println(currClass+" is subclass of about: "+sub);
						if ( onto.containsKey ( currClass ) ) { onto.remove ( currClass ); }
						if ( debug_op ) System.out.println("putting: "+currClass+","+sub);
						onto.put(currClass, sub);
					}


				}

			} else if ( qName == "rdfs:subClassOf" ) {
				// defining subclasses next
				String sub;
				if ( ( sub = attrs.getValue("rdf:resource") ) != null ) {
					sub = rmH(sub);
					if ( onto.containsKey ( currClass ) ) { onto.remove ( currClass ); }
					if ( debug_op ) System.out.println("putting: "+currClass+","+sub);
					onto.put(currClass, sub);
				}
				subclass = true;
			}
	}

	/**
	 * This method completes the action of startElement, namely unsets references that has been set, and adding items to misc lists.
	 * @see #startElement(String, String, String, Attributes)
	 */
	public void endElement(String namespaceURI, String sName, String qName ) throws SAXException {
			if ( qName == "owl:Class" ) {
				currClass = null;
			} else if ( qName == "rdfs:subClassOf" ) {
				subclass = false;
			}
   }

	/**
	 * The string between tags (usually empty) is processed by this method. It is only used to to set PMID of abstracts.
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		String s = new String(buf, offset, len);
		s = s.trim();
  	}

	public void emit(String s) throws SAXException {
		if( s != null ) System.out.println(s);
	}

	/**
	 * This method removes a hash (#) occuring at the start of a string.
	 * @param s the string
	 * @return the same string, except for an eventual hash
	 */
	public String rmH(String s){
		if( s.startsWith("#") ) {
			return s.substring(1);				
		} else {
			return s;
		}
	}

}
