import java.util.ArrayList;

/**
 * Container class to hold data on TEST-abstracts. There are two arrays: tqls that contains all tql-statements corresponding to sentences in this abstract, and events that contains all events related to sentences/TQL-statements in this abstract.
 */
public class TQLAbstract {

	public ArrayList<TQL> tqls = new ArrayList<TQL>();
	public ArrayList<Event> events = new ArrayList<Event>();
	public String medlineid;

	/**
	 * An abstract can be constructed without any additional parameters.
	 */
	public TQLAbstract(){

	}

	/**
	 * Method to add a TQL to the list of TQL-statements in this abstract.
	 * @param t the tql to be added
	 */
	public void addTQL(TQL t){
		tqls.add(t);
	}

	/**
	 * Method to add an Event to the list of events in this abstract.
	 * @param e the event to be added
	 */
	public void addEvent(Event e){
		events.add(e);
	}

	/**
	 * Method to set the identification string of this abstract.
	 * @param s the identification string
	 */
	public void setID(String s) {
		medlineid = s;
	}

	/**
	 * Method to access the logic statements in this abstract.
	 * @return arraylist of statements in this abstract
	 */
	public ArrayList getTQL(){
		return tqls;
	}

	/**
	 * Method to access the events in this abstract.
	 * @return arraylist of events in this abstract.
	 */
	public ArrayList getEvents(){
		return events;
	}

	/**
	 * Method to count number of events in this abstact.
	 * @return number of events in this abstract
	 */
	public int countEvents(){
		return events.size();
	}

	/**
	 * Method to count number of logic statements in this abstact.
	 * @return number of logic statements in this abstract
	 */
	public int countTQLs(){
		return tqls.size();
	}

}
