import java.util.ArrayList;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * This class handles the content of XML in TEST-files. For each tag, there is a definition of action to take. This is usually related to putting the attribute data of the tag somewhere.
 * @see TQLParser
 */
public class TQLHandler extends DefaultHandler {

	public OntologyParser op;
	public TQLAbstract tqlabstract;
	public ArrayList<TQLAbstract> abstracts = new ArrayList<TQLAbstract>();
	public boolean inBlock = false;
	public TQL t;

	/**
	 * A TQLHandler can be constructed without any particular parameters
	 */
	public TQLHandler(){
	}

	/**
	 * Method to set  the ontologyparser this Handler should use.
	 * @param op an ontologyparser
	 */
	public void setOntologyParser(OntologyParser op){
		this.op = op;
	}

	/**
	 * Invoked when a document is about to be parsed.\ Does nothing.
	 */
	public void startDocument(){
		// System.out.println("START");
	}

	/**
	 * Invoked when a document is finished parsed. Does nothing but informing the user about this happening.
	 */
	public void endDocument(){
		System.out.println("== tql parsing complete");
	}

	/**
	 * Called when the parser find a new starting tag. This tag is matched against a number of clauses, i.e. sentence, term, event .. If no mathcing clause is found, the tag is ignored.
	 * This method sets data according to which tag has been discovered.
	 * @see #endElement(String, String, String)
	 */
	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {

		int input,block = 0;

		if(qName == "tql"){

			inBlock = true;

			input = (new Integer(attrs.getValue("input"))).intValue();
			block = (new Integer(attrs.getValue("block"))).intValue();

			t = new TQL(input, block, tqlabstract, op);

		} else if( qName == "abstract" ) {

			String medlineid = attrs.getValue("medlineid");

			tqlabstract = new TQLAbstract(); 
			tqlabstract.setID(medlineid);
		}
   }

	/**
	 * This method completes the action of startElement, namely unsets references that has been set, and adding items to misc lists.
	 * @see #startElement(String, String, String, Attributes)
	 */
	public void endElement(String namespaceURI, String sName, String qName ) throws SAXException {
		if( qName == "tql") {
			t.extractEvents();
			tqlabstract.addTQL(t);
			inBlock = false;

		} else if ( qName == "abstract" ) {
				abstracts.add(tqlabstract);
				tqlabstract = null;
		}
   }

	/**
	 * The string between tags (usually empty) is processed by this method. It is only used to to set PMID of abstracts.
	 */
	public void characters(char buf[], int offset, int len) throws SAXException {
		String s = new String(buf, offset, len);
		s = s.trim();
		
		if(inBlock && (s.length() > 0)){

			// TQL t = (TQL) tql.get(tql.size());
			t.addLine(s);

		}
		/* code for printing elements
        	String s = new String(buf, offset, len);
		s = s.trim();
        	if(s.length() > 0) emit("CHAR: |"+s+"|");
		*/
    	}

	public void emit(String s) throws SAXException {
		System.out.println(s);
	}

	/**
	 * Accessor method to get the abstracts that has been handled by this Handler.
	 * @return the abstracts processes by this handler
	 */
	public ArrayList getAbstracts(){
		return abstracts;
	}

}
