
import java.util.ArrayList;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * This class invokes the parsing of a TEST-document.
 * @see TQLHandler
 */
public class TQLParser {

	public File file;
	//public ArrayList tqlBlocks = new ArrayList();
	public TQLHandler handler;
	public OntologyParser op;

	public TQLParser(File file, OntologyParser op){
		this.file = file;
		this.op = op;
		parseTQL(file);
	}

	/**
	 * Creates a new parser, and starts this with the actual document.
	 * @param file the xml-file containing the test corpus
	 */
	public void parseTQL(File file){

		try {
			handler = new TQLHandler();
			handler.setOntologyParser(op);
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse( file , handler);
		} catch ( Exception e ) {
			e.printStackTrace();
			System.exit(1);
		}

	}

	/**
	 * Access method to get the handler of this parser.
	 * @return the handler of this document
	 */
	public DefaultHandler getHandler(){
		return handler;
	}

	/**
	 * Access method to get the abstracts parsed by this parser.
	 * @return a list of abstracts parsed by this parser
	 */
	public ArrayList getAbstracts(){
		return handler.getAbstracts();
	}


}
