import java.io.*;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Iterator;

/**
 * This class takes care of writing distilled XML-events to the output-files given as parameters on the command line.
 *
 */
public class XMLWriter {

	public String tqlxml_name;
	public File tqlxml_file;
	public FileWriter tqlxml_fw;

	public String genxml_name;
	public File genxml_file;
	public FileWriter genxml_fw;

	/**
	 * To create this Writer, it needs pathname to one or both files to be written.
	 * @param gen the golden destillation file
	 * @param tql the test destillation file
	 */
	public XMLWriter(String gen, String tql) {
		this.genxml_name = gen;
		this.tqlxml_name = tql;

		if ( tqlxml_name != null ){

				try {
					tqlxml_file = new File(tqlxml_name);
					tqlxml_fw = new FileWriter(tqlxml_file,false);	
					tqlxml_fw.write("<root>\n");
			
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to tql xml file: "+tqlxml_name);
				}
		}

		if ( genxml_name != null ) {

				try {
					genxml_file = new File(genxml_name);
					genxml_fw = new FileWriter(genxml_file,false);	
					genxml_fw.write("<root>\n");
			
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to gen xml file: "+genxml_name);
				}

		}

	}

	/**
	 * Write a TQL/TEST-event to the output files.
	 * @param e the event to be written
	 */
	public void writeTQLEvent(Event e){

			if ( tqlxml_name != null ) { 
				try {
					tqlxml_fw.write(e.getXML());
					tqlxml_fw.flush();
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to tql xml file: "+tqlxml_name);
				}
			}
	}

	/**
	 * Write a GENIA/GOLD-event to the output files.
	 * @param e the event to be written
	 */
	public void writeGENEvent(GEvent e){

		   if ( genxml_name != null ) {
				try {
					genxml_fw.write(e.getXML());
					genxml_fw.flush();
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to gen xml file: "+genxml_name);
				}
			}
	}

	/**
	 * Flush the writer and close the output files.
	 */
	public void closeXML(){

			if ( tqlxml_name != null ) {
				try {
					tqlxml_fw.write("</root>\n");
					tqlxml_fw.flush();
					tqlxml_fw.close();
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to tql xml file: "+tqlxml_name);
				}
			}

			if ( genxml_name != null ) {
				try {
					genxml_fw.write("</root>\n");
					genxml_fw.flush();
					genxml_fw.close();
				} catch ( Exception ex ) {
					ex.printStackTrace();
					System.err.println(" == WARNING: Could not write to gen xml file: "+genxml_name);
				}
			}


	}


	/*
	 * Write essential GENIA events to file if filename is given
	 *

	if ( genxml_name != null ) {
			String gex = "<root>\n";	  
			for ( GAbstract g : genia ) {
				ArrayList<GEvent> evs = g.getEvents();
				for ( GEvent e : evs ) {
					gex += e.getXML();	
				}	  
			}
			gex += "</root>";	

			try {

				File genxml_file = new File(genxml_name);
				FileWriter genxml_fw = new FileWriter(genxml_file,false);	
				genxml_fw.write(gex);
				genxml_fw.flush();
				genxml_fw.close();


			} catch ( Exception e ) {

				e.printStackTrace();
				System.err.println(" == WARNING: Could not write to tql xml file: "+tqlxml_name);
			}
	}
	 */


}
